#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <libgen.h>
#include "file-manager.h"
#include "file-system.h"
#include "select-file.h"
#include "../curses2.h"
#include "../lib.h"
#include "../libw.h"

/**
* @rturn Inode number aktualniho adresare, nebo 0
*/
static ino_t getInoNumber(fileInfo * info)
{
    int i = 0;
    int len;
    if (!info)
	return 0;
    while (strcmp(info[i].path, "")) {
	len = strlen(info[i].path);
	if (!strcmp(info[i].path + len - 2, "/.")) {
	    return info[i].stat.st_ino;
	}
	i++;
    }
    return 0;
}

static int getIxForIno(ino_t ino, fileInfo * info)
{
    int i = 0;
    if (!info)
	return 0;
    if (!ino)
	return 0;		// nekdy muze byt na FS soubor s neplatnym ino! (viz primountovany win adresar ve VirtualBoxu)
    while (strcmp(info[i].path, "")) {
	if (info[i].stat.st_ino == ino) {
	    return i;
	}
	i++;
    }
    return 0;

}

char *fileManager(char *dir)
{
    static char file[PATH_MAX];
    char *validDir;
    fileInfo *info;
    int i, len;
    ino_t oldino, ino = 0;
    WINDOW *win = newwin2(FM_LINES, FM_COLS, 1, 1);
#ifdef _FM_TEST
    wrefresh2(stdscr);
#else
    pridej_do_priority_w(1, win, stdscr, LINES, 0);
#endif
    if (!win)
	return NULL;
    if (dir == NULL) {
	getcwd(file, PATH_MAX);
    } else {
	strncpy(file, dir, PATH_MAX);
    }
    validDir = getValidDirectory(file);
    if (!validDir) {
#ifndef _FM_TEST
	odeber_z_priority(win);
#endif
	delwin2(win);
	return NULL;
    }

    do {
	validDir = getValidDirectory(file);
	info = getDirectoryFiles(validDir);
	i = getIxForIno(ino, info);
	i = selectFile(win, validDir, info, i);
	ino = 0;
	if (i == -1) {
	    /* navart z neplatneho adresare */
	    strcpy(file, dirname(validDir));
	    ino = oldino;
	    oldino = 0;
	    i = 0;
	    continue;
	}
	if (i >= 0) {
	    len = strlen(info[i].path);
	    if (!strcmp(info[i].path + len - 3, "/..")) {
		strcpy(file, dirname(validDir));
		ino = getInoNumber(info);
		oldino = 0;
	    } else if (!strcmp(info[i].path + len - 2, "/.")) {
		// nedelej nic
	    } else {
		strncpy(file, info[i].path, PATH_MAX);
		oldino = info[i].stat.st_ino;
	    }
	    if (!info[i].is_directory) {
#ifndef _FM_TEST
		odeber_z_priority(win);
#endif
		delwin2(win);
		free(info);
		return file;
	    }
	}
	if (info)
	    free(info);
    } while (i >= 0);
#ifndef _FM_TEST
    odeber_z_priority(win);
#endif
    delwin2(win);
    return NULL;
}
